@@ -84,6 +84,14 @@ gint           act_user_collate                   (ActUser   *user1,
                                                    ActUser   *user2);
 gboolean       act_user_is_loaded                 (ActUser   *user);
 
+void           act_user_get_password_expiration_policy (ActUser   *user,
+                                                        gint64    *expiration_time,
+                                                        gint64    *last_change_time,
+                                                        gint64    *min_days_between_changes,
+                                                        gint64    *max_days_between_changes,
+                                                        gint64    *days_to_warn,
+                                                        gint64    *days_after_expiration_until_lock);
+
 void           act_user_set_email                 (ActUser    *user,
                                                    const char *email);
 void           act_user_set_language              (ActUser    *user,
